%  
% Compositional Zoning in Garnet and its Modification by diffusion (CZGM)
% ========================================================================= 
% CZGM is a set of Matlab functions for numerical modelling of garnet
% formation under variable pressure (P) and temperature (T) conditions.
% Simulation proceeds along a user defined PT-path. Potential for garnet
% growth is based on modal proportion (GV) and equilibrium composition of
% garnet basic elements (Fe, Mn, Mg, Ca) computed in fractionation mode of
% the Perple_X program of Connolly (2005). Available material is
% distributed in garnet population of prescribed size-distribution, whereas
% garnets grow by spherical shells of constant radius (Gaidies et al.,
% 2007). Multicomponent diffusion is solved by the finite difference method
% (Crank, 1975) with several options for diffusion coefficients
% (Chakraborty and Ganguly, 1992; Carlson, 2006; Vielzeuf, 2007) and for
% boundary conditions. Diffusion proceeds both during garnet growth and
% after it when garnet follows remaining part of the PT-path.
% 
% CZGM runs (was debugged) under MATLAB Version R2016b. Its use is for
% research is free but without guarantee. Comments to its mathematical and
% software aspects are welcomed on jezek@natur.cuni.cz.
%
% In case of using CZGM please cite the article 
% Faryad, S,W., Jezek, J., 2019. Compositional zoning in garnet and its
% modification by diffusion during pressure and temperature changes in
% metamorphic rocks; an approach and software. LITHOS, Volumes 332333,
% Pages 287-295. 
% https://doi.org/10.1016/j.lithos.2019.03.002.
% =========================================================================
%
% Modelling procedure
%  
% Modelling with CZGM consists of several steps:
% (0) Preparation and initial setup
% (1) Design of PT-path
% (2) Fractionation along PT path
% (3) Formation of garnet population
% (4) Assignment of time to PT path
% (5) Multicomponent diffusion in a chosen garnet
% 
% User can run the whole sequence of steps, or run steps 1,2 and then
% repeatedly 3 and 5 with different choice of garnet and subsequent
% diffusion, or repeat diffusion, step 5, with different time setup 4.
%   
%  
% (0) Preparation and initial setup 
% 
% CZGM uses two folders, CODE and DATA. The folder CODE contains all Matlab
% functions and the Perple_X program vertex.exe and its auxiliary files.
% User shouldn't make changes in this folder. By a session, CODE is Matlab
% working folder and all the functions run here, typically from command line
% (see  CZGM_script). The folder DATA contains files described bellow.
%
% For fractionation (step 2), the Perple_X program vertex.exe must be
% present in CODE folder. It will use 2 files that must be prepared in the
% DATA folder: 
% (1) Build file gtfract.dat describing fractionation along a PT-path 
% (calculation type 10). User creates it by build.exe or by editing
% and older file. Indices of 1st & 2nd independent component must be 1
% and 2 (in this order). Ranks of Fe, Mn, Mg and Ca in the thermodynamic
% component list, as they appear in gtfract.dat have to be written in
% parameters iFe, iMn, iMg, iCa of the function initial_setup. In
% gtfract.dat, coordinate file name PT_coord.dat must be mentioned. 
% (2) Garnet model enters via the file gtpipe.txt.
%  
% To ensure an optimal PT-path design (step 1), it is recommended to
% prepare by Perple_X (vertex.exe and werami.exe) 5 files with PT-grids of
% garnet modal proportion Vol.tab, and composition Fe.tab, Mn.tab, Mg.tab,
% Ca.tab. These will be displayed by the function make_path to show the
% potential for garnet formation and to facilitate PT-path design. The
% build file for it is the same as gtfract.dat, only gridded mode is given
% (calculation type 5). The structure of *.tab files must correspond to
% reading procedure in the function get_tab that is included in the
% function make_path.
%  
% Basic option to design PT-path is by mouse. Nevertheless, user can supply
% a coordinate file my_PT_path.dat in the folder DATA. Its structure must
% correspond to reading procedure in the function make_path (input from
% file) or user must change the code appropriately.
%  
% After completing mentioned files, user runs the function initial_setup
% providing initial definitions of parameters (see its code for their
% description).
%  
%  
% (1) Design of PT-path 
%  
% Design of PT-path is provided by the function make_path. The function
% displays PT-grids prepared in step 0, if they exist (PT_design>0),
% showing the isoplets of equilibrium compositions of garnet basic elements
% (Fe, Mn, Mg, Fe, Ca) and the garnet modal proportion (GV). Over one of
% these graphs (which is given by the value of the parameter PT_design),
% user designs a PT-path by mouse picking. The path is interpolated in a
% predefined number of points Pi and Ti that will be used in fractionation
% (step 2). Interpolated profiles (Mni, Mgi, Fei, Cai and GVi) are shown
% that provide a first look on potential for garnet development. Time (ti)
% is formally added to each PT-point to assure unique interpolation of
% complex PT-paths. If files with PT-grids are not available, user sets
% PT_design=0, and PT-path design proceeds on a blank PT-area. Optionally,
% the PT-path may input from file my_PT_path.dat that is prepred in the
% folder DATA and user sets the parameter iq_file=1. PT-coordinates and
% formal time that were prepared/processed by make_path are written in
% global variables Pi, Ti and ti to be accessible for other functions.
% 
%  
% (2) Fractionation along PT-path
%  
% The function run_vertex runs the Perplex program vertex.exe for
% fractionation along prescribed PT-path. The program vertex.exe is called
% in batch mode, reading its input parameters from the files gtfract.dat
% and gtpipe.txt that must be prepared in the folder DATA. PT-coordinates
% that were prepared by make_path as global variables Pi and Ti are
% automatically written in a coordinate file that is passed to vertex.exe.
% Program vertex writes its fractionation results about garnet and
% bulk-rock composition in two files. The function run_vertex reads these
% data and eventually cleans it (omits rows with NaNs and zeros).
% Fractionation summary is plotted. For a comparison, the garnet
% composition resulting from fractionation is plotted over curves obtained
% by interpolation of PT-grids. This allows to compare the cases with and
% without fractionation. On output, run_vertex provides a set of PTt-points
% (PG, TG, tG) where garnet is formed its cummulative amount normed to one
% (GV) and normed concentrations of the basic elements (FeG, MnG, MgG,
% CaG). Times tG(1) and tG(end) show where first and last garnet were
% created and in between it equals to corresponding equidistant values of
% ti. Cumulative curve of the increase of garnet volume (GV as a
% function of tG) will be used for modelling garnet formation in the
% function make_garnets (step 3). Function run_vertex also provides
% information about the part of PT-path that remains to go after garnet
% stops growing (after tG(end) which will be used in make_diffusion (step
% 5).
%  
%  
% (3) Formation of garnet population
%  
% The function make_garnets takes information about fractionation along the
% selected PT-path to model a population of garnets that are created and
% grow gradually along the PT-path so that they, at the end, reach a
% prescribed size-distribution. It implies that nucleation of garnet is
% modeled as an episodic process that is controlled not only by the
% fractionation data but also by an assumption about size-distribution. The
% procedure is similar although technically not identical to Gaidies et al.
% (2008). User defines garnet size distribution by means of a function f(r)
% that is then discretized from rmin to rmax into a number of classes which
% defines also constant radius increment dr. Garnets are approximated by
% spheres increasing by shells of thickness dr. From a previous run of the
% function run_vertex, the function make_garnets takes the amount of garnet
% volume accumulated along the PT-path as a function of relative time,
% GV(t). This function is normalized to 1, i.e., it goes from a small
% volume GV at the PT-point where vertex predicted first appearance of
% garnet, to GV=1 at the PT-point where the garnet formation stopped.
% Make_garnets runs a cycle through all size-classes. For each size-class
% of garnet, a point on the cumulative GV(t) curve is found where the
% volume increase is sufficient to feed a newly created class and to
% increase all existing classes by new shells of thickness dr. This
% procedure ensures that, at the PT-point where garnet formation stops, all
% size-classes are built proportionally to the required size-distribution.
% The assignment of formal (relative) time to the classes allows also to
% assign the corresponding P, T and modal composition to individual shells
% in each garnet. Growth of all garnet classes is tracked graphically but
% only information about garnet whose diffusional relaxation user intends
% to model subsequently (by means of the function  make_diffusion) is kept
% by the end of the run of make_garnets and passed to global variables.
% This garnet must be specified by user before running make_garnets by
% setting the parameter garnet_no equal to the class-number where the
% garnet belongs. The parameter garnet_no is predefined already in the
% initial_setup, but can be changed ad hoc from the command line anytime
% before running make_garnets. The size of the selected garnet which was
% relative up to now is rescaled according to the parameter r_rescaled. 
% On the output, the function make_garnets provides profiles of garnet
% composition (variables Fer, Mnr, Mgr, Car and corresponding time tr)
% altogether with information about time remaining to go along the PT-path
% after garnet stopped its growing. These data will be used in
% make_difussion.
%
%  
% (4) Assignment of time to PT path
%  
% Until this step, time scale assigned to the PT-path which was defined in
% make_path (step 1) had only auxiliary role in the interpolation.
% Nevertheless, real time is important for diffusion (step 5). Before
% applying diffusion, user should decide whether actual time scale
% corresponds to his intentions or specify it otherwise. This is can be
% done interactively by the function make_time, by mouse-picking of several
% points on the PT-path and inputting corresponding time from keyboard.
% Time scale that will be used in the modelling of diffusion is then
% rescaled accordingly. For a quick enlargement or shortening of previously
% defined time scale, there is no need to run make_time. It is sufficient
% to change the multiplicative factor time_fac from the command line. For
% example setting time_fac=10 means that the actual time scale will be
% multiplied by 10. In the initial_setup, default time_fac=1 is predefined.
% Time is given in My.
%  
%  
% (5) Multicomponent diffusion in a chosen garnet
%  
% Multicomponent diffusion in garnet is modeled using the FicksOnsager
% approach (Onsager, 1945) as a coupled system o three diffusion equations
% in spherical coordinate system fixed in the center of garnet. Equations
% are numerically solved by means of finite difference method. Diffusion
% coefficients  depend on tracer diffusion coefficients according to
% (Lasaga, 1979). The application of diffusion on a garnet consists of two
% parts. First the function make_diffusion follows, step by step, the
% garnet growth computed and recorded previously in make_garnets and at
% each step it runs diffusion in the actual garnet for time interval
% dt=tr(i)-tr(i-1). Dirichlet boundary condition is applied at the
% outermost node keeping constant concentrations of Fe, Mn, Mg, and Ca on
% the garnet rim. Second, when garnet is completed, diffusion continues
% along the remaining PT-path until its end. Boundary condition for this
% part is optional and can be set from command line by means of the
% parameter BC. There are 3 possibilities: BC=0  keep constant
% concentrations of the elements on the garnet rim;  BC=1 - user supplied
% boundary concentrations; BC=2 - insulated garnet (closed system with zero
% gradient on the rim); BC=3 - boundary concentrations of Fe and Mg are
% interpolated from PT-grids (files Fe.tab, Mg.tab) at given points of the
% remaining PT-path, while concentration of Mn=0, and Ca=1-Fe-Mg. Beside
% the options for boundary conditions, the function make_diffusion provides
% also several options for diffusion coefficients that are available via
% the parameter diff_mode (diff_mode=1 for Chakraborty and Ganguly, 1992; 2
% for Carlson, 2006; 3 for Vielzeuf, 2007).
%  
%  
% CZGM folders
%  
% CZGM uses two folders, CODE and DATA. All Matlab functions of CZGM
% (initial_setup, make_path, run_vertex, make_garnets, make_time,
% make_diffusion) should be in the folder CODE that is set up as the Matlab
% current folder. A modelling session consist of repeated calling of CZGM
% functions (see CZGM_script) that share important variables in the form
% of globals. Folder CODE must contain the program vertex.exe and files
% hp11ver.dat, solution_model.dat needed for its functioning. Outputs from
% vertex.exe are automatically written in the CODE folder. User should not
% change the content of this folder.
%  
% Folder DATA contains files and data provided by user, namely project
% files gtfract.dat and gtpipe.txt defining the fractionation which will be
% used by the function run_vertex. Other files in DATA are optional. Highly
% recommended is to prepare files with modal proportion and equilibrium
% composition of garnet basic elements as gridded outputs from vertex.exe
% transformed by werami.exe (files Vol.tab, Fe.tab, Mn.tab, Mg.tab,
% Ca.tab). Folder DATA may also contain an Excel file with experimental
% garnet profiles that can be displayed in the functions make_garnets and
% make_diffusion. During modelling, some of mentioned files are
% automatically copied from DATA to CODE folder. Although the basic option
% is to create the PT-path by mouse, folder DATA may also contain a user
% file defining the PT-path. Decision which variant (mouse-path) is given
% by the parameter iq_input.
%  
%  
% Example of the application 
% 
% Copy folders CODE and DATA, supply vertex.exe into CODE and follow 
% CZGM_script.
%  
%  

